module WTOSC
  # Class that contains a serie of calls required for spaceship to execute the
  class Executor
    def initialize(queries, options = {}, before: [], after: [])
      raise ArgumentError if queries.empty?

      percona_commands = commands(queries, options)

      @after = after
      @commands = before.concat(percona_commands, after)
    end

    def execute!(run_commands = @commands)
      run_commands.each_with_index do |command, index|
        command.call(self)
        success "Finished step (#{index + 1}/#{run_commands.size})"
      end
    end

    def clean_up!
      warning 'Cleaning up...'
      execute!(@after)
    end

    def success(message)
      return if ENV['RACK_ENV'] == 'test'

      puts "\e[32m\n#{message}\e[0m"
    end

    def warning(message)
      return if ENV['RACK_ENV'] == 'test'

      puts "\e[33m\n#{message}\n---\e[0m"
    end

    def suppress
      return '' unless ENV['RACK_ENV'] == 'test'

      " > /dev/null"
    end

    private

    def commands(queries, options)
      queries.flat_map do |query|
        full_query = ShellCommand.build!(query.merge(options))

        [
          WTOSC::PerconaCommand.new(full_query + '--dry-run'),
          WTOSC::PerconaConfirmer,
          WTOSC::PerconaCommand.new(full_query + '--execute')
        ]
      end
    end
  end
end
