class ShellCommand
  BASE_COMMAND =
    'pt-online-schema-change '\
      '--user %{username} '\
      '--host %{host} '\
      '--alter "%{query}" '\
      'D=%{database},t=%{table} '

  DEFAULTS = { pause_file: true, progress: true }
  MANDATORY_KEYS = [:username, :host, :query, :database, :table]

  def self.build!(options)
    new(options).build
  end

  def initialize(options)
    @options = options.merge(DEFAULTS)
    @base_query = BASE_COMMAND % @options
  end

  def build
    @options.each_pair do |key, value|
      next if MANDATORY_KEYS.include?(key)

      value = ENV[key.to_s] || value

      part = case key
             when :password
               "--password #{value} "
             when :pause_file
               "--pause-file=pause.txt "
             when :progress
               "--progress percentage,1 "
             when :max_load
               "--max-load Threads_running=#{value} "
             when :critical_load
               "--critical-load Threads_running=#{value} "
             when :innodb_lock_wait_timeout
               "--set-vars innodb_lock_wait_timeout=#{value} "
             else
               raise ArgumentError, "unknown option #{key}"
             end

      @base_query.insert(-1, part)
    end

    @base_query
  end
end
